;---------------------------------------------------
;----- Create Agents -------------------------------
;---------------------------------------------------
breed [rabbits rabbit]
rabbits-own [ energy ]


;---------------------------------------------------
;----- Setup ---------------------------------------
;---------------------------------------------------
; Observer Procedure
to setup
  clear-all
  ask patches [grow-grass]
  
  set-default-shape rabbits "rabbit"
  create-rabbits initial-num-rabbits 
  [ set color white
    setxy random-xcor random-ycor
    set energy random 10  ;start with a random amt. of energy
  ]
  
  reset-ticks
end


;---------------------------------------------------
;----- Go ------------------------------------------
;---------------------------------------------------
; Observer Procedure
to go
  if not any? rabbits [ stop ]
  
  ask patches [grow-grass]
  
  ask rabbits
  [ move
    eat-grass
    reproduce
    death 
  ]
  
  tick
end


;---------------------------------------------------
;----- Grow Grass ----------------------------------
;---------------------------------------------------
; Patch Procedure
to grow-grass 
  if (pcolor = black) and (random-float 1 < grass-grow-rate) [ set pcolor green ]
end


;---------------------------------------------------
;----- Move ----------------------------------------
;---------------------------------------------------
; Rabbit Procedure
to move  
  set heading heading + (random-float 100 - 50) 
  fd 1
  set energy energy - 0.5  ;; moving takes some energy
end


;---------------------------------------------------
;----- Eat Grass -----------------------------------
;---------------------------------------------------
; Rabbit Procedure
to eat-grass
  ;; gain energy by eating grass
  if pcolor = green
  [ set pcolor black
    set energy energy + grass-energy 
  ]
end


;---------------------------------------------------
;----- Reproduce -----------------------------------
;---------------------------------------------------
; Rabbit Procedure
to reproduce     ;; rabbit procedure
  ;; give birth to a new rabbit, but it takes lots of energy
  if energy > birth-threshold
  [ set energy (energy / 2)
    hatch 1 [ fd 1 ] 
  ]
end


;---------------------------------------------------
;----- Death ---------------------------------------
;---------------------------------------------------
; Rabbit Procedure
to death 
  ;; die if you run out of energy
  if energy < 0 [ die ]
end
@#$#@#$#@
GRAPHICS-WINDOW
215
10
717
533
20
20
12.0
1
10
1
1
1
0
1
1
1
-20
20
-20
20
1
1
1
ticks
30.0

BUTTON
39
385
94
418
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
104
385
159
418
go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
10
279
200
312
grass-grow-rate
grass-grow-rate
0.0
0.05
0.026
0.001
1
NIL
HORIZONTAL

SLIDER
8
194
202
227
grass-energy
grass-energy
0.0
10.0
5
0.5
1
NIL
HORIZONTAL

SLIDER
10
32
205
65
initial-num-rabbits
initial-num-rabbits
0.0
500.0
150
1.0
1
NIL
HORIZONTAL

SLIDER
10
111
204
144
birth-threshold
birth-threshold
0.0
20.0
5
1.0
1
NIL
HORIZONTAL

PLOT
728
10
1228
206
Populations
Time
Pop
0.0
100.0
0.0
111.0
true
true
"set-plot-y-range 0 initial-num-rabbits" ""
PENS
"grass" 1.0 0 -10899396 true "" "plot count patches with [pcolor = green]"
"rabbits" 1.0 0 -12895429 true "" "plot count rabbits"

MONITOR
1177
66
1266
111
count rabbits
count rabbits
1
1
11

TEXTBOX
14
13
209
31
Set the initial number of rabbits:
11
0.0
1

TEXTBOX
13
79
217
107
Set the minimum amount of energy required for a rabbit to reproduce:
11
0.0
1

TEXTBOX
10
160
202
193
Set the amount of energy contained in each patch of grass:
11
0.0
1

TEXTBOX
13
246
206
274
Set the growth rate at which new patches of grass appear:
11
0.0
1

TEXTBOX
10
316
205
365
Example: A value of 0.015 means that at each time step each patch that does not currently contain a patch of grass has a 1.5% chance of growing a patch of grass.
9
3.0
1

BUTTON
58
442
137
475
MovieGo
setup\nmovie-start \"Rabbits.mov\"\nrepeat 100\n[ go\n  movie-grab-view ]\nmovie-close
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

@#$#@#$#@
# Purpose
This agent-based model explores a simple ecosystem made up of rabbits and grass. The rabbits wander around randomly, and the grass grows randomly.   When a rabbit bumps into a patch of grass, it eats the grass and gains energy. If the rabbit gains enough energy, it reproduces. Each rabbit loses energy every time it moves and reproduces. If a rabbits loses too much energy it dies.


# Entities and Variables
## Entities and their State Variables
There are two entities in this model: rabbits (agents), and patches. Each rabbit has variables for their location, heading, and energy level. Each patch has a color variable which indicates if it contains grass or not.

## Global Parameters
All of the global variables are set by the user in the interface.

  * `initial-num-rabbits` is the number of rabbits which initial appear in the ABM "world".
  * `birth-threshold` is the minimum amount of energy required for a rabbit to reproduce.
  * `grass-energy` is the amount of energy a single patch of grass contains.
  * `grass-grow-rate` gives the probability that a single patch which currently does not contain grass will produce grass in a single time step.

# Process Overview
After the model is initialized (see Initialization section for details), a simulation will run until there are no rabbits left or until the user stops the simulation. During each time step, the following events occur in the order they are listed:

  * If there are no rabbits left the simulation stops
  * Each patch executes the Grow Grass procedure (see Submodels section for details)
  * Each rabbit excutes the following procedures: Move > Eat Grass > Reproduce > Death
  * The time step is incremented by 1

## Submodels
### Grow Grass ![Patch Context](http://ccl.northwestern.edu/netlogo/docs/images/patch.gif)
If the patch is currently colored black (no grass), then with a probability of `grass-growth-rate` the patch will be colored green (contain grass). 

Note, to select when to color a patch green based on the probability `grass-growth-rate`, a random floating point value (real number) between 0 and 1 is generated. If that value is less than `grass-growth-rate`, then the patch is colored green, otherwise it remains black.

### Move ![Turtle Context](http://ccl.northwestern.edu/netlogo/docs/images/turtle.gif)
The rabbit sets is current heading to a randomly selected heading within 50 degress (left or right). 

### Eat Grass ![Turtle Context](http://ccl.northwestern.edu/netlogo/docs/images/turtle.gif)
If the patch the rabbit currently occupies is green (has grass), then the rabbit will eat the grass. In doing so that patch color is turned to black (no more grass), and the rabbit's energy variable increases by the amount of `grass-energy`.

### Reproduce ![Turtle Context](http://ccl.northwestern.edu/netlogo/docs/images/turtle.gif)
If the rabbit's energy variable is greater than the value of `birth-threshold`, then the rabbit will reproduce. In do so, the rabbit loses half of its energy. The offspring is place one patch length away from its parent in the direction of the parent's current heading.

### Death ![Turtle Context](http://ccl.northwestern.edu/netlogo/docs/images/turtle.gif)
If the rabbit's energy variable is less than 0, then the rabbit dies.

# How to Run a Simulation

1. Set each of the global variables at the desired values. 
2. Click on the SETUP button to initialize the simulation: `initial-num-rabbits` will be place randomly across the model world, and be given a random initial energy (integer) value in the range [0,9]. The model world starts with no grass (all patches black), but during the setup procedure creates patches of grass according to the Grow Grass submodel.
3. Click on the GO button to start the simulation.

# ...
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

rabbit
false
0
Circle -7500403 true true 76 150 148
Polygon -7500403 true true 176 164 222 113 238 56 230 0 193 38 176 91
Polygon -7500403 true true 124 164 78 113 62 56 70 0 107 38 124 91

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.2.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
